@@ -4,7 +4,7 @@ use 5.006;
 use strict;
 use warnings;
 
-our $VERSION = '0.08';
+our $VERSION = '0.09';
 
 =head1 NAME
 
@@ -12,7 +12,7 @@ Crypt::LE - Let's Encrypt API interfacing module.
 
 =head1 VERSION
 
-Version 0.08
+Version 0.09
 
 =head1 SYNOPSIS
 
@@ -50,7 +50,7 @@ for an example of a challenge-handling plugin.
 
 Crypt::LE is shipped with a self-sufficient client for obtaining SSL certificates - le.pl. Usage example:
 
- B<le.pl --key account.key --csr domain.csr --csr-key domain.key --crt domain.crt --domains "www.domain.ext,domain.ext" --generate-missing>
+B<le.pl --key account.key --csr domain.csr --csr-key domain.key --crt domain.crt --domains "www.domain.ext,domain.ext" --generate-missing>
 
 That will generate an account key and a CSR (plus key) if they are missing. If any of those files exist, they will just be loaded, so it is safe to re-run
 the client. Run le.pl without any parameters or with C<--help> to see more details and usage examples.
@@ -59,13 +59,13 @@ The provided client also supports challenge-handling plugins, such as L<Crypt::L
 plugins, such as L<Crypt::LE::Complete::Simple>. You can easily handle challenges and trigger specific actions when your certificate gets issued
 by using those modules as templates, without modifying the client code. You can also pass custom parameters to your modules from le.pl command line:
 
- le.pl ... --handle-with Crypt::LE::Challenge::Simple --handle-params '{"key1": 1, "key2": "one"}'
+B<le.pl ... --handle-with Crypt::LE::Challenge::Simple --handle-params '{"key1": 1, "key2": "one"}'>
  
- le.pl ... --complete-with Crypt::LE::Complete::Simple --complete-params '{"key1": 1, "key2": "one"}'
+B<le.pl ... --complete-with Crypt::LE::Complete::Simple --complete-params '{"key1": 1, "key2": "one"}'>
  
 The parameters don't have to be put directly in the command line, you could also give a name of a file containing valid JSON to read them from.
 
- le.pl ... --complete-params complete.json
+B<le.pl ... --complete-params complete.json>
  
 Crypt::LE::Challenge:: and Crypt::LE::Complete:: namespaces are suggested for new plugins.
  
@@ -609,10 +609,12 @@ The account key fingerprint
 =back
 
 The type of the challenge accepted is optional and it is 'http' by default. The following values are currently available: 'http', 'tls', 'dns'. 
-New values which might be added by Let's Encrypt will be supported automaticaly. On error you can call failed_domains() method, which returns 
-an array reference to domain names for which the challenge was not accepted successfully.
+New values which might be added by Let's Encrypt will be supported automaticaly. While currently all domains being processed share the same type
+of challenge, it might be changed in the future versions.
 
-The callback shoud return a true value on success.
+On error you can call failed_domains() method, which returns an array reference to domain names for which the challenge was not accepted successfully.
+
+The callback should return a true value on success.
 
 The callback could be either a code reference (for example to a subroutine in your program) or a blessed reference to a module handling
 the challenge. In the latter case the module should have methods defined for handling appropriate challenge type, such as:
@@ -681,19 +683,75 @@ sub accept_challenge {
     return $self->_status(OK, $domains_accepted ? "Accepted challenges for $domains_accepted domain(s)." : "There are no domains for which challenges need to be accepted.");
 }
 
-=head2 verify_challenge()
+=head2 verify_challenge([$callback] [, $params] [, $type])
 
 Asks Let's Encrypt server to verify the results of the challenge. On error you can call failed_domains() method, which returns an array reference to domain names 
 for which the challenge was not verified successfully.
 
-Returns: OK | ERROR.
+Optionally you can set up a callback, which will be called for each domain with the results of verification. The callback will receive two parameters -
+a hash reference with the results and a hash reference of parameters optionally passed to verify_challenge(). The results data has the following keys:
+
+=over 14
+
+=item C<domain>
+
+The domain name processed (lower-case)
+
+=item C<token>
+
+The challenge token
+
+=item C<fingerprint>
+ 
+The account key fingerprint
+
+=item C<valid>
+ 
+Set to 1 if the domain has been verified successfully or set to 0 otherwise.
+
+=back
+
+The type of the challenge accepted is optional and it is 'http' by default. The following values are currently available: 'http', 'tls', 'dns'.
+
+The callback should return a true value on success.
+
+The callback could be either a code reference (for example to a subroutine in your program) or a blessed reference to a module handling
+the verification outcome. In the latter case the module should have methods defined for handling appropriate verification type, such as:
+
+=over
+
+=item
+
+B<handle_verification_http()>
+
+=item
+
+B<handle_verification_tls()>
+
+=item
+
+B<handle_verification_dns()>
+
+=back
+
+You can use L<Crypt::LE::Challenge::Simple> example module as a template.
+
+Returns: OK | INVALID_DATA | ERROR.
 
 =cut
 
 sub verify_challenge {
     my $self = shift;
+    my ($cb, $params, $type) = @_;
     return $self->_status(ERROR, "Domains and challenges need to be set before verifying.") unless ($self->{domains} and $self->{challenges});
     return $self->_status(OK, "There are no active challenges to verify") unless $self->{active_challenges};
+    my $mod_callback = ($cb and blessed $cb) ? 1 : 0;
+    $type||='http';
+    my $handler = "handle_verification_$type";
+    if ($cb) {
+        return $self->_status(INVALID_DATA, "Valid callback has not been provided.") unless ($cb and ((ref $cb eq 'CODE') or ($mod_callback and $cb->can($handler))));
+        return $self->_status(INVALID_DATA, "Passed parameters are not pointing to a hash.") if ($params and (ref $params ne 'HASH'));
+    }
     my ($domains_verified, @domains_failed);
     foreach my $domain (sort keys %{$self->{domains}}) {
         unless (defined $self->{domains}->{$domain} and !$self->{domains}->{$domain}) {
@@ -708,7 +766,7 @@ sub verify_challenge {
         my $type = delete $self->{active_challenges}->{$domain};
         my $token = $self->{challenges}->{$domain}->{$type}->{token};
         my ($status, $content) = $self->_request($self->{challenges}->{$domain}->{$type}->{uri}, { resource => 'challenge', keyAuthorization => "$token.$self->{fingerprint}" });
-        my $validated = 0;
+        my ($validated, $cb_reset) = (0, 0);
         if ($status == ACCEPTED) {
             if ($content->{uri}) {
                 my $check = $content->{uri};
@@ -719,17 +777,31 @@ sub verify_challenge {
                 if ($status == ACCEPTED and $content and $content->{status}) {
                     if ($content->{status}=~/^(?:in)?valid$/) {
                         if ($content->{status} eq 'valid') {
-                            $self->_debug("Domain $domain has been verified succesfully.");
+                            $self->_debug("Domain $domain has been verified successfully.");
                             $self->{domains}->{$domain} = 1;
-                            $domains_verified++;
                             $validated = 1;
                         }
                     }
                 }
             }
         }
-        unless ($validated) {
-            $self->_debug("Domain $domain has failed verification (status code $status).", $content);
+        if ($cb) {
+            my $rv;
+            my $callback_data = { domain => $domain, token => $self->{challenges}->{$domain}->{$type}->{token}, fingerprint => $self->{fingerprint}, valid => $validated };
+            eval {
+                $rv = $mod_callback ? $cb->$handler($callback_data, $params) : &$cb($callback_data, $params); 
+            };
+            if ($@ or !$rv) {
+                # NB: Error in callback will propagate, even if validation process returned OK.
+                $self->_debug("Verification callback for domain $domain " . ($@ ? "thrown an error: $@" : "did not return a true value"));
+                $cb_reset = 1 if $validated;
+                $validated = 0;
+            }
+        }
+        if ($validated) {
+            $domains_verified++;
+        } else {
+            $self->_debug("Domain $domain has failed verification (status code $status).", $content) unless $cb_reset;
             push @domains_failed, $domain;
         }
     }
