@@ -3,12 +3,12 @@ use Data::Dumper;
 use strict;
 use warnings;
 
-our $VERSION = '0.07';
+our $VERSION = '0.09';
 
 =head1 NAME
 
 Crypt::LE::Challenge::Simple - A boilerplate for extending Crypt::LE and Crypt::LE client
-application (le.pl) with challenge handlers.
+application (le.pl) with challenge/varification handlers.
 
 =head1 SYNOPSIS
 
@@ -19,11 +19,12 @@ application (le.pl) with challenge handlers.
  my $simple_challenge = Crypt::LE::Challenge::Simple->new();
  ..
  $le->accept_challenge($simple_challenge);
+ $le->verify_challenge($simple_challenge);
 
 =head1 DESCRIPTION
 
 Crypt::LE provides the functionality necessary to use Let's Encrypt API and generate free SSL certificates for your domains.
-This Crypt::LE plugin is an example of how challenge handling can be done by an external module.
+This Crypt::LE plugin is an example of how challenge and verification handling can be done by an external module.
 
 B<This module can also be used with the provided Crypt::LE client application - le.pl:>
 
@@ -36,10 +37,11 @@ sub new { bless {}, shift }
 sub handle_challenge_http {
     my $self = shift;
     my ($challenge, $params) = @_;
+    print __PACKAGE__, "\n";
     if ($params) {
         print "Parameters passed: " . Dumper($params);
     }
-    print "Domain: $challenge->{domain} expects '$challenge->{token}.$challenge->{fingerprint}' text in '$challenge->{token}' file\n";
+    print "Domain $challenge->{domain} expects '$challenge->{token}.$challenge->{fingerprint}' text in '$challenge->{token}' file\n";
     print "Press <Enter>";
     <STDIN>;
     return 1;
@@ -55,6 +57,27 @@ sub handle_challenge_dns {
     die "Oh, this challenge is not supported!\n";
 }
 
+sub handle_verification_http {
+    my $self = shift;
+    my ($verification, $params) = @_;
+    print __PACKAGE__, "\n";
+    if ($params) {
+        print "Parameters passed: " . Dumper($params);
+    }
+    print "Domain $verification->{domain} " . ($verification->{valid} ? "has been verified successfully." : "has NOT been verified.") . "\n";
+    print "You can now delete '$verification->{token}' file\n";
+    <STDIN>;
+    return 1;
+}
+
+sub handle_verification_tls {
+    1;
+}
+
+sub handle_verification_dns {
+    1;
+}
+
 =head1 AUTHOR
 
 Alexander Yezhov, C<< <leader at cpan.org> >>
