@@ -9,7 +9,7 @@ use Crypt::LE ':errors';
 sub info { print shift, "\n" }
 sub bail { &info; exit 255 }
 
-my $VERSION = '0.08';
+my $VERSION = '0.09';
 
 my ($account_key, $csr, $csr_key, $domains, $challenge_path, $crt, $live, $generate_missing, $generate_only, 
     $handle_with, $handle_as, $handle_params, $complete_with, $complete_params, $revoke, $verified, $debug, $help);
@@ -179,7 +179,7 @@ $le->accept_tos(); # No need to check explicitly for tos_changed() - Crypt::LE w
 unless ($verified) {
     bail $le->error_details if $le->request_challenge();
     bail $le->error_details if $le->accept_challenge($handler ? $handler : \&process_challenge, $handle_params, $handle_as);
-    bail $le->error_details if $le->verify_challenge;
+    bail $le->error_details if $le->verify_challenge($handler ? $handler : \&process_verification, $handle_params, $handle_as);
 }
 info "Requesting domain certificate";
 bail $le->error_details if $le->request_certificate();
@@ -269,3 +269,12 @@ EOF
  	<STDIN>;
 	return 1;
 };
+
+sub process_verification {
+    my $results = shift;
+    info  "Domain verification results for '$results->{domain}': " . ($results->{valid} ? "success." : "error.");
+    my $file = $challenge_path ? "$challenge_path/$results->{token}" : $results->{token};
+    # NB: We don't delete challenge files automatically, this option might be added later.
+    info "You can now delete '$file' file.";
+    1;
+}
